import time
import requests
from bs4 import BeautifulSoup
import ray

@ray.remote
def bs_fetch():
    session = requests.Session()
    data = session.get("https://www.business-standard.com/")
    soup = BeautifulSoup(data.content, "html.parser")

    links = []
    titles = []
    descs = []
    imgs = []

    a_data = soup.find(class_ = "row-inner topB")
    a_tags = a_data.find_all("a")
    topics = ["economy-policy", "companies"]
    for a in a_tags:
        try:
            topic = a["href"].split("/")[2]
        except:
            continue

        if a["href"][:9] == "/article/" and a["href"][-5:] == ".html" and topic in topics:
            links.append(f'https://www.business-standard.com{a["href"]}')
            titles.append(a.text)

    for index, link in enumerate(links):
        if titles[index] == "\n\n":
            links.remove(link)
            titles.pop(index)
            continue
    
    for index, link in enumerate(links):
        article = session.get(link)
        article_soup = BeautifulSoup(article.content, "html.parser")

        try:
            desc = article_soup.find("h2", attrs = {"class", "alternativeHeadline"}).text
            descs.append(desc)

            img = article_soup.find("img", attrs = {"class", "imgCont"})
            if not img:
                imgs.append(None)
                continue
            imgs.append(img["src"])
        except:
            try:
                links.remove(link)
                titles.pop(index)
            except:
                continue
        time.sleep(0.01)

    data = []
    for index, link in enumerate(links):
        data.append({
            "title": titles[index],
            "desc": descs[index],
            "link": links[index],
            "img": imgs[index]
        })

    for i in data:
        if not i["img"]:
            data.remove(i)

    if len(data) > 5:
        data = data[:5]

    return data


import time
import requests
from bs4 import BeautifulSoup
import ray

@ray.remote
def et_fetch():
    session = requests.Session()
    data = session.get("https://economictimes.indiatimes.com/markets")
    soup = BeautifulSoup(data.content, "lxml")

    links = []
    titles = []
    descs = []
    imgs = []

    a_data = soup.find(id = "topStories")
    a_tags = a_data.find_all("a")
    count = 0
    for a in a_tags:
        count += 1
        if a["href"][:21] == "/markets/stocks/news/":
            links.append(f'https://economictimes.indiatimes.com{a["href"]}')
            titles.append(a.text)
        if count == 5:
            break
    
    for index, link in enumerate(links):
        article = session.get(link)
        article_soup = BeautifulSoup(article.content, "lxml")

        try:
            desc = article_soup.find_all("h2", attrs = {"class", "summary"})[0].text
            descs.append(desc)

            img = article_soup.find_all("img")
            imgs.append(img[3]["src"])
        except:
            links.remove(link)
            titles.pop(index)
        time.sleep(0.01)

    data = []
    for index, link in enumerate(links):
        data.append({
            "title": titles[index],
            "desc": descs[index],
            "link": link,
            "img": imgs[index]
        })
    
    if len(data) > 5:
        data = data[:5]
    
    return data


from .mc_fetcher import mc_fetch
from .et_fetcher import et_fetch
from .bs_fetcher import bs_fetch
import ray

class Fetcher:
    def __init__(self):
        self.data = self.fetch()

    def fetch(self):
        data = []

        mc = mc_fetch.remote()
        et = et_fetch.remote()
        bs = bs_fetch.remote()
        mc_data, et_data, bs_data = ray.get([mc, et, bs])
        for i in mc_data:
            data.append(i)
        for i in et_data:
            data.append(i)
        for i in bs_data:
            data.append(i)
        
        return data


import time
import requests
from bs4 import BeautifulSoup
import ray

@ray.remote
def mc_fetch():
    session = requests.Session()
    data = requests.get("https://www.moneycontrol.com/")
    soup = BeautifulSoup(data.content, "lxml")
    time.sleep(0.01)

    links = []
    titles = []
    descs = []
    imgs = []

    a_data = soup.find(class_ = "sub-col-left")
    a_tags = a_data.find_all("a")
    last_link = ""
    for a in a_tags:
        if (
            a["href"] != last_link
            and a["href"][28:43] == "/news/business/"
            and a["href"][42:51] != "/markets/"
            and a["href"][42:55] != "/commodities/"
        ):
            last_link = a["href"]
            links.append(a["href"])
            titles.append(a["title"])

    for index, link in enumerate(links):
        article = session.get(link)
        article_soup = BeautifulSoup(article.content, "lxml")

        try:
            desc = article_soup.find("h2", attrs = {"class", "article_desc"}).text
            descs.append(desc)

            img_data = article_soup.find(class_ = "article_image")
            img = img_data.find_all("img")
            imgs.append(img[0]["data-src"])
        except:
            try:
                links.remove(link)
                titles.pop(index)
            except:
                pass
        time.sleep(0.01)
    
    data = []
    for index, link in enumerate(links):
        try:
            data.append({
                "title": titles[index],
                "desc": descs[index],
                "link": link,
                "img": imgs[index]
            })
        except:
            break
    
    if len(data) > 5:
        data = data[:5]

    return data